from django.test import TestCase
from locks.models import Lock
from locks.locks import DbLock, LockOffError, LockOnError
from locks.decorators import lock_and_log
import _thread



class LockTestCase(TestCase):

    def setUp(self):
        lock = Lock.objects.create(name="A")
        lock.save()

    def test_pre_created_lock(self):
        """Test locking behaviour of precreated lock"""
        lock = DbLock(name="A")

        lock.release(hard=True)

        # test if concurrent process is stopped
        lock.set()

        self.assertRaises(LockOnError, lock.set)

        # check if incorrect releasing of locks raises errors
        lock.release()

        self.assertRaises(LockOffError, lock.release)

    def test_auto_lock_creation(self):
        lock = DbLock("B")
        lock.set()
        self.assertRaises(LockOnError, lock.set)

class LockAndLogTestCase(TestCase):

    def test_decorator(self):
        import logging
        logger = logging.getLogger(__name__).disabled

        @lock_and_log(logger=logger, uid='id')
        def lock_me():
            return True

        res = lock_me()
        self.assertTrue(res)


class TestLockAtomicity(TestCase):
    def test_lock(self):
        lock = DbLock(name='D')

        def do():
            lock.set()
            lock.release()

        for i in range(5):
            _thread.start_new_thread(do, ())


    def test_decorator(self):
        import logging
        logger = logging.getLogger(__name__)


        @lock_and_log(logger, uid='G')
        def do():
            print('heyhey')

        for i in range(5):
            _thread.start_new_thread(do, ())


from locks.locks import DbLock
import time


def lock_and_log(logger, uid, raise_error=True, sleep=0):
    """
    decorator to create a unique lock for a function, log the function and release the lock if an exception occurs.
    if raise_error is set to false, it will simple exit the function (handy for if you don't want to log)
    if sleep is set, it will sleep for x seconds before releasing the lock
    """
    def decorator(func):
        def wrapper(*args, **kwargs):
            lock_name = func.__name__ + __name__ + uid
            lock = DbLock(lock_name)
            lock.set()
            try:
                res = func(*args, **kwargs)
                time.sleep(sleep)
                lock.delete()
                return res
            except Exception:
                time.sleep(sleep)
                lock.delete()
                logger.exception('error in {}'.format(func.__name__))
                return None
        return wrapper
    return decorator


from django.db import models


class Lock(models.Model):
    name = models.CharField(blank=True, max_length=100, unique=True)
    locked = models.BooleanField(default=False)
    created_at = models.DateTimeField(auto_now=True)
    expire = models.BooleanField()
    expiration = models.DateTimeField()

    def __str__(self):
        return self.name

    class Meta:
        ordering = ['-created_at', 'expiration']
        get_latest_by = '-created_at'


from django.core.management.base import BaseCommand, CommandError
from locks.locks import DbLock


class Command(BaseCommand):
    help = 'start/stop a TxBroadcaster'

    def add_arguments(self, parser):
        parser.add_argument('uid', nargs=1, type=int)

    def handle(self, *args, **options):
        self.stdout.write('unlocking: {uid}'.format(
            uid=options['uid'][0]))
        DbLock(name=options['uid'][0]).release(hard=True)
        self.stdout.write('unlocked!')






# Generated by Django 2.0 on 2018-03-15 20:23

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Lock',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, max_length=100, unique=True)),
                ('locked', models.BooleanField(default=False)),
                ('created_at', models.DateTimeField(auto_now=True)),
                ('expire', models.BooleanField()),
                ('expiration', models.DateTimeField()),
            ],
        ),
    ]



from setuptools import setup

setup(
    name='django-db_locks',
    version='0.0.3',
    packages=['locks', 'locks.management', 'locks.management.commands', 'locks.migrations'],
    url='https://github.com/BlockHub/django-locks',
    license='',
    author='Charles',
    author_email='karel@blockhub.nl',
    description='easy databaselocking for processes',
    install_requires=['django', 'psycopg2-binary']
)
